package com.mdd.admin.service.impl;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.mdd.admin.cache.RedisCache;
import com.mdd.admin.service.ILicenseService;
import com.mdd.admin.validate.commons.PageValidate;
import com.mdd.admin.validate.license.LicenseCreateValidate;
import com.mdd.admin.validate.license.LicenseSearchValidate;
import com.mdd.admin.validate.license.LicenseUpdateValidate;
import com.mdd.admin.vo.LicenseDetailVo;
import com.mdd.admin.vo.LicenseListedVo;
import com.mdd.common.core.AjaxResult;
import com.mdd.common.core.PageResult;
import com.mdd.common.entity.License;
import com.mdd.common.mapper.LicenseMapper;
import com.mdd.common.util.HttpUtils;
import com.mdd.common.util.JwtUtils;
import com.mdd.common.util.StringUtils;
import com.mdd.common.util.TimeUtils;
import io.jsonwebtoken.Claims;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.TimeUnit;
import javax.annotation.PostConstruct;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

@Service
public class LicenseServiceImpl implements ILicenseService {
   @Autowired
   LicenseMapper licenseMapper;
   @Autowired
   private RedisCache redisCache;
   @Value("${like.saas-url}")
   public String saasUrl;
   private Boolean checking;
   private final int licenseExpireTime;

   public LicenseServiceImpl() {
      this.checking = Boolean.FALSE;
      this.licenseExpireTime = 60;
   }

   public AjaxResult<?> checkLicense() {
      String license = (String)this.redisCache.getCacheObject("license_key");

      try {
         Claims claims = JwtUtils.parseToken(license);
         if (claims == null) {
            return AjaxResult.failed("许可证过期，请去saas平台查看商户状态！");
         }
      } catch (Exception var3) {
         return AjaxResult.failed("许可证过期，请去saas平台查看商户状态！");
      }

      return AjaxResult.success();
   }

   @PostConstruct
   public AjaxResult<?> getLicense() {
      AjaxResult var8;
      try {
         if (this.checking) {
            AjaxResult var17 = AjaxResult.success("上一次请求还未完成，稍后再试");
            return var17;
         }

         this.checking = Boolean.TRUE;
         QueryWrapper<License> queryWrapper = new QueryWrapper();
         queryWrapper.orderByDesc("license_id");
         IPage<License> iPage = this.licenseMapper.selectPage(new Page(1L, 1L), queryWrapper);
         if (iPage.getRecords() == null || iPage.getRecords().isEmpty()) {
            this.checking = Boolean.FALSE;
            AjaxResult var19 = AjaxResult.failed("未查询到license");
            return var19;
         }

         License license = (License)iPage.getRecords().get(0);
         String result = HttpUtils.sendGet(this.saasUrl + "/system/external/license?code=" + license.getCode());
         if (StringUtils.isEmpty(result)) {
            Integer retryCount = (Integer)this.redisCache.getCacheObject("license_err_cnt_key");
            AjaxResult var21;
            if (retryCount != null && retryCount >= 5) {
               this.redisCache.deleteObject("license_key");
               this.redisCache.deleteObject("license_key_period");
               this.checking = Boolean.FALSE;
               var21 = AjaxResult.failed("连续5次【获取动态许可证】请求失败，禁止使用");
               return var21;
            }

            this.redisCache.incrementCounter("license_err_cnt_key");
            var21 = AjaxResult.success("", "【获取动态许可证】请求失败，稍后再试");
            return var21;
         }

         JSONObject json = JSONObject.parseObject(result);
         Integer code = json.getInteger("code");
         if (code != null && code == 200) {
            JSONObject data = json.getJSONObject("data");
            String token = data.getString("token");
            String period = data.getString("period");
            this.redisCache.setCacheObject("license_key", token, 60, TimeUnit.MINUTES);
            this.redisCache.setCacheObject("license_key_period", period, 60, TimeUnit.MINUTES);
            this.redisCache.deleteObject("license_err_cnt_key");
            AjaxResult var10 = AjaxResult.success();
            return var10;
         }

         Integer retryCount = (Integer)this.redisCache.getCacheObject("license_err_cnt_key");
         if (retryCount != null && retryCount >= 5) {
            this.redisCache.deleteObject("license_key");
            this.redisCache.deleteObject("license_key_period");
            this.checking = Boolean.FALSE;
            var8 = AjaxResult.failed("连续5次【获取动态许可证】失败，原因：" + json.getString("msg"));
            return var8;
         }

         this.redisCache.incrementCounter("license_err_cnt_key");
         System.out.println("【获取动态许可证】失败，原因：" + json.getString("msg"));
         var8 = AjaxResult.success("【获取动态许可证】失败，原因：" + json.getString("msg"));
      } catch (Exception var14) {
         Exception e = var14;
         AjaxResult var2 = AjaxResult.failed("【获取动态许可证】异常：" + e.getMessage());
         return var2;
      } finally {
         this.checking = Boolean.FALSE;
      }

      return var8;
   }

   public PageResult<LicenseListedVo> list(PageValidate pageValidate, LicenseSearchValidate searchValidate) {
      Integer page = pageValidate.getPageNo();
      Integer limit = pageValidate.getPageSize();
      QueryWrapper<License> queryWrapper = new QueryWrapper();
      queryWrapper.orderByDesc("license_id");
      this.licenseMapper.setSearch(queryWrapper, searchValidate, new String[]{"=:code:str", "=:checkUrl@check_url:str"});
      IPage<License> iPage = this.licenseMapper.selectPage(new Page((long)page, (long)limit), queryWrapper);
      List<LicenseListedVo> list = new LinkedList();
      Iterator var8 = iPage.getRecords().iterator();

      while(var8.hasNext()) {
         License item = (License)var8.next();
         LicenseListedVo vo = new LicenseListedVo();
         BeanUtils.copyProperties(item, vo);
         vo.setCreateTime(TimeUtils.timestampToDate(item.getCreateTime()));
         list.add(vo);
      }

      return PageResult.iPageHandle(iPage.getTotal(), iPage.getCurrent(), iPage.getSize(), list);
   }

   public LicenseDetailVo detail(Integer id) {
      License model = (License)this.licenseMapper.selectOne((Wrapper)((QueryWrapper)(new QueryWrapper()).eq("license_id", id)).last("limit 1"));
      Assert.notNull(model, "数据不存在");
      LicenseDetailVo vo = new LicenseDetailVo();
      BeanUtils.copyProperties(model, vo);
      return vo;
   }

   public void add(LicenseCreateValidate createValidate) {
      License model = new License();
      model.setCode(createValidate.getCode());
      model.setCheckUrl(createValidate.getCheckUrl());
      model.setCreateTime(TimeUtils.nowDate());
      this.licenseMapper.insert(model);
   }

   public void edit(LicenseUpdateValidate updateValidate) {
      License model = (License)this.licenseMapper.selectOne((Wrapper)((QueryWrapper)(new QueryWrapper()).eq("license_id", updateValidate.getLicenseId())).last("limit 1"));
      Assert.notNull(model, "数据不存在!");
      model.setLicenseId(updateValidate.getLicenseId());
      model.setCode(updateValidate.getCode());
      model.setCheckUrl(updateValidate.getCheckUrl());
      this.licenseMapper.updateById(model);
   }

   public void del(Integer id) {
      License model = (License)this.licenseMapper.selectOne((Wrapper)((QueryWrapper)(new QueryWrapper()).eq("license_id", id)).last("limit 1"));
      Assert.notNull(model, "数据不存在!");
      this.licenseMapper.delete((Wrapper)(new QueryWrapper()).eq("license_id", id));
   }
}
